@@ -29,9 +29,6 @@
 #include "interface.h"
 #include "asciiselect.h"
 
-#include <gconf/gconf-client.h>
-#include <gnome.h>
-
 void
 cb_about_click (GtkWidget *widget, gpointer user_data)
 {
@@ -44,101 +41,99 @@ cb_about_click (GtkWidget *widget, gpointer user_data)
 	    NULL
     };
     /* Translator credits */
-    gchar *translator_credits = _("translator_credits");
+    gchar *translator_credits = _("");
     GtkWidget *dialog;
-    GdkPixbuf *logo  = NULL;
-    GError    *error = NULL;
-    gchar     *logo_fn;
+    GdkPixbuf *logo = NULL;
+    gchar *logo_fn;
 
-    logo_fn = gnome_program_locate_file (NULL, GNOME_FILE_DOMAIN_PIXMAP, "gnome-character-map.png", FALSE, NULL);
-    logo = gdk_pixbuf_new_from_file (logo_fn, &error);
-    
-    if (error) {
-    	    g_warning (G_STRLOC ": cannot open %s: %s", logo_fn, error->message);
-	    g_error_free (error);
+    logo_fn = gnome_pixmap_file("gcharmap-logo.png");
+    if (logo_fn != NULL)
+    {
+	    logo = gdk_pixbuf_new_from_file(logo_fn, NULL /* error */);
+	    g_free(logo_fn);
     }
-    
-    g_free (logo_fn);
 
     dialog = gnome_about_new (
-      _("GNOME Character Map"),
+      _("Gnome Character Map"),
       VERSION,
       "Copyright (c) 2000 Hongli Lai",
-      _("Select, copy and paste characters from your font "
-	"into other applications"),
+      _("The Gnome equalivant of Microsoft Windows' Character Map. "
+      "Warning: might contain bad English."),
       authors,
       (const char **)documenters,
-      strcmp (translator_credits, "translator_credits") != 0 ? translator_credits : NULL,
+      (const char *)translator_credits,
       logo
     );
-    
-    if (logo) {
-    	    gdk_pixbuf_unref (logo);
-    }
     gtk_widget_show (dialog);
 }
 
-void 
-cb_browsebtn_click (GtkButton *button, gpointer data)
-{
-	cb_insert_char_click (NULL, data);
-}
-
 
 void
 cb_charbtn_click (GtkButton *button, gpointer user_data)
 {
     GtkLabel *label = GTK_LABEL (GTK_BIN (button)->child);
-    const gchar *text;
-    gint current_pos;
-    
-    text = gtk_label_get_text (label);
+    gchar *text;
 
-    gtk_editable_get_position(GTK_EDITABLE(mainapp->entry));
-    gtk_editable_insert_text (GTK_EDITABLE (mainapp->entry), text,
-			      strlen (text), &current_pos);
-    gtk_editable_set_position (GTK_EDITABLE (mainapp->entry), current_pos + 1);
-/*
+    gtk_label_get (label, &text);
+
+    if (strcmp (text, _("del")) == 0) {
+	    if ( ! mainapp->insert_at_end) {
+		    GtkEditable *editable = GTK_EDITABLE (mainapp->entry);
+		    /* snarfed from GTK+
+		     * -George */
+		    gint sel_start, sel_end;
+
+		    gtk_editable_get_selection_bounds(editable, &sel_start,
+				    &sel_end);
+		    if (sel_start != sel_end) {
+			    gtk_editable_delete_selection (editable);
+		    } else {
+			    gint old_pos = gtk_editable_get_position(editable);
+
+			    if (gtk_editable_get_position(editable) < -1)
+				    gtk_editable_set_position(editable, 0);
+			    else
+				    gtk_editable_set_position(editable, gtk_editable_get_position(editable) + 1);
+
+			    gtk_editable_delete_text (editable, old_pos, gtk_editable_get_position(editable));
+		    }
+	    }
+    } else if ( ! mainapp->insert_at_end) {
+	    gint current_pos = gtk_editable_get_position(GTK_EDITABLE(mainapp->entry));
+	    gtk_editable_insert_text (GTK_EDITABLE (mainapp->entry), text,
+				      strlen (text), &current_pos);
     } else {
-           gint current_pos;
-           gtk_editable_set_position (GTK_EDITABLE (mainapp->entry), -1);
-           current_pos = gtk_editable_get_position (GTK_EDITABLE (mainapp->entry));
-           gtk_editable_insert_text (GTK_EDITABLE (mainapp->entry), text, strlen(text), &current_pos);
-           gtk_editable_set_position (GTK_EDITABLE (mainapp->entry), current_pos + 1);
-    } */
+	    gtk_entry_append_text (GTK_ENTRY (mainapp->entry), text);
+    }
 }
 
 
-gboolean
-cb_charbtn_enter (GtkButton *button, GdkEventFocus *event, gpointer user_data)
+void
+cb_charbtn_enter (GtkButton *button, gpointer user_data)
 {
     GtkLabel *label = GTK_LABEL (GTK_BIN (button)->child);
-    gchar *s;
+    gchar *text, *s;
    int code;
-    const gchar *text;
 
-    text = gtk_label_get_text (label);
+    gtk_label_get (label, &text);
     if (strcmp (text, _("del")) == 0) {
 	    code = 127;
     } else {
-            code = g_utf8_get_char(text);
+	    code = (unsigned char)text[0];
     }
 
     s = g_strdup_printf (_(" %s: Character code %d"), text, code);
     gnome_appbar_set_status (GNOME_APPBAR (GNOME_APP (mainapp->window)->statusbar), s);
+    gtk_label_set_text (GTK_LABEL (mainapp->preview_label), text);
     g_free (s);
-    return FALSE;
 }
 
 
-gboolean
-cb_charbtn_leave (GtkButton *button, GdkEventFocus *event, gpointer user_data)
+void
+cb_charbtn_leave (GtkButton *button, gpointer user_data)
 {
     gnome_appbar_pop (GNOME_APPBAR (GNOME_APP (mainapp->window)->statusbar));
-#if 0    
     gtk_label_set_text (GTK_LABEL (mainapp->preview_label), NULL);
-#endif
-    return FALSE;
 }
 
 
@@ -152,16 +147,7 @@ cb_clear_click (GtkWidget *widget, gpointer user_data)
 void
 cb_copy_click (GtkWidget *widget, gpointer user_data)
 {
-    gint start, end;
-    gboolean selection_flag;
-
-    selection_flag = gtk_editable_get_selection_bounds (GTK_EDITABLE (mainapp->entry), &start, &end);
-
-    if(selection_flag)
-        gtk_editable_select_region (GTK_EDITABLE (mainapp->entry), start, end);
-    else
-        cb_select_all_click (widget, user_data);
-
+    cb_select_all_click (widget, user_data);
     gtk_editable_copy_clipboard (GTK_EDITABLE (mainapp->entry));
     gnome_app_flash (GNOME_APP (mainapp->window), _("Text copied to clipboard..."));
 }
@@ -170,15 +156,10 @@ cb_copy_click (GtkWidget *widget, gpointer user_data)
 void
 cb_cut_click (GtkWidget *widget, gpointer user_data)
 {
-    gint start, end;
-    gboolean selection_flag;
-
-    selection_flag = gtk_editable_get_selection_bounds (GTK_EDITABLE (mainapp->entry), &start, &end);
-
-    if(selection_flag)
-        gtk_editable_select_region (GTK_EDITABLE (mainapp->entry), start, end);
-    else
-        cb_select_all_click (widget, user_data);
+    cb_select_all_click (widget, user_data);
+    while (gtk_events_pending ()) gtk_main_iteration ();
+    usleep (500000);
+    while (gtk_events_pending ()) gtk_main_iteration ();
     gtk_editable_cut_clipboard (GTK_EDITABLE (mainapp->entry));
     gnome_app_flash (GNOME_APP (mainapp->window), _("Text cut to clipboard..."));
 }
@@ -210,8 +191,10 @@ cb_fontpicker_font_set (GnomeFontPicker *gfp, gchar *font_name)
 void
 cb_help_click (GtkWidget *widget, gpointer user_data)
 {
-    GError *error = NULL;
-    gnome_help_display("gnome-character-map",NULL,&error);
+    gnome_help_display (
+        gnome_program_get_human_readable_name(gnome_program_get ()),
+		    "index.html",
+		    NULL /* error */);
 }
 
 
@@ -221,6 +204,8 @@ cb_insert_char_click (GtkWidget *widget, gpointer user_data)
     AsciiSelect *ascii_selector;
 
     ascii_selector = ascii_select_new ();
+    gnome_dialog_set_parent (GNOME_DIALOG (ascii_selector->window),
+      GTK_WINDOW (mainapp->window));
     gtk_widget_show (ascii_selector->window);
 }
 
@@ -236,7 +221,26 @@ cb_paste_click (GtkWidget *widget, gpointer user_data)
 void
 cb_select_all_click (GtkWidget *widget, gpointer user_data)
 {
-    gtk_editable_select_region (GTK_EDITABLE (mainapp->entry), 0, -1);
+    gtk_entry_select_region (GTK_ENTRY (mainapp->entry), 0, -1);
+}
+
+
+void
+cb_set_button_focusable (GtkCheckMenuItem *checkmenuitem, gpointer user_data)
+{
+    guint i;
+
+    if (checkmenuitem->active == TRUE)
+    {
+        for (i = 0; i < g_list_length (mainapp->buttons); i++)
+            GTK_WIDGET_SET_FLAGS (GTK_WIDGET (g_list_nth_data (
+              mainapp->buttons, i)), GTK_CAN_FOCUS);
+    } else
+    {
+        for (i = 0; i < g_list_length (mainapp->buttons); i++)
+            GTK_WIDGET_UNSET_FLAGS (GTK_WIDGET (g_list_nth_data (
+              mainapp->buttons, i)), GTK_CAN_FOCUS);
+    }
 }
 
 
@@ -247,4 +251,41 @@ cb_set_chartable_font (GtkWidget *widget, gpointer user_data)
 }
 
 
+void
+cb_set_insert_at_end (GtkCheckMenuItem *checkmenuitem, gpointer user_data)
+{
+    mainapp->insert_at_end = checkmenuitem->active;
+}
+
+
+void
+cb_toggle_actionbar (GtkCheckMenuItem *checkmenuitem, gpointer user_data)
+{
+    if (checkmenuitem->active == TRUE)
+        gtk_widget_show (mainapp->actionbar);
+    else
+        gtk_widget_hide (mainapp->actionbar);
+}
+
+
+void
+cb_toggle_textbar (GtkCheckMenuItem *checkmenuitem, gpointer user_data)
+{
+    if (checkmenuitem->active == TRUE)
+        gtk_widget_show (mainapp->textbar);
+    else
+        gtk_widget_hide (mainapp->textbar);
+}
+
+
+void
+cb_toggle_statusbar (GtkCheckMenuItem *checkmenuitem, gpointer user_data)
+{
+    if (checkmenuitem->active == TRUE)
+        gtk_widget_show (GNOME_APP (mainapp->window)->statusbar);
+    else
+        gtk_widget_hide (GNOME_APP (mainapp->window)->statusbar);
+}
+
+
 #endif /* _CALLBACKS_C_ */
