@@ -109,7 +109,7 @@ static gint lwidth=0 , rwidth=0;
 
 static void clist_add(GtkCList *cl, char *l, char *r)
 {
-	GdkFont * font = gtk_widget_get_style(GTK_WIDGET(cl))->font; 
+	GdkFont * font = gtk_style_get_font(GTK_WIDGET(cl)->style); 
 	gchar *data[2];
 	
 	data[0]=l;
@@ -139,6 +139,7 @@ int dialog_checklist(const char *title, const char *prompt, int height, int widt
 					GNOME_STOCK_BUTTON_CANCEL, NULL);
 		GtkWidget *hbox, *vbox, *sw;
 
+		gnome_dialog_set_default(GNOME_DIALOG(w), 0);
 		gnome_dialog_set_close(GNOME_DIALOG(w), TRUE);
 		gtk_window_set_title(GTK_WINDOW(w), title);
 
