@@ -1,4 +1,4 @@
-/* $Id: gdict-app.c,v 1.17.2.1 2001/10/03 03:38:06 jfleck Exp $ */
+/* $Id: gdict-app.c,v 1.28 2001/12/05 22:38:30 kevinv Exp $ */
 /* -*- mode: c; style: k&r; c-basic-offset: 4 -*- */
 
 /*
@@ -23,12 +23,12 @@
 #include <string.h>
 #include <ctype.h>
 
+#define GTK_ENABLE_BROKEN
 #include <gnome.h>
 
 #include "dict.h"
 #include "gdict-about.h"
 #include "gdict-pref.h"
-#include "gdict-applet.h"
 
 #include "gdict-app.h"
 #include "gdict-defbox.h"
@@ -89,6 +89,8 @@ gdict_init_context (void)
     
     context = dict_context_new (gdict_pref.server, gdict_pref.port);
     context->command = dict_disconnect_command_new ();
+    
+    defbox->context = context;
 
     if (context->hostinfo)
 	return 0;
@@ -102,7 +104,7 @@ gdict_app_clear (void)
     gchar *word;
     
     gdict_defbox_clear (defbox);
-    
+  
     /* Update entry */
     if ((word = gdict_defbox_get_word (defbox))) {
         gtk_entry_set_text (GTK_ENTRY(word_entry), word);
@@ -146,8 +148,6 @@ gdict_open_speller (void)
 {
     if (!speller) {
         speller = GDICT_SPELLER (gdict_speller_new (context));
-        gnome_dialog_set_parent (GNOME_DIALOG (speller), 
-                                 GTK_WINDOW (gdict_app));
         
         if (!speller) return;
 
@@ -163,7 +163,8 @@ gdict_open_speller (void)
                             GTK_SIGNAL_FUNC (spell_close_cb), NULL);
     }
     
-    gtk_widget_show (GTK_WIDGET (speller));
+    if (!GTK_WIDGET_VISIBLE (GTK_WIDGET (speller)))
+    	gtk_widget_show (GTK_WIDGET (speller));
 }
 
 
@@ -216,9 +217,11 @@ gdict_app_do_lookup (gchar *text)
 static void
 lookup_entry (void) 
 {
-    gchar *text = gtk_entry_get_text (GTK_ENTRY (word_entry));
+    gchar *text = gtk_editable_get_chars (GTK_EDITABLE (word_entry), 0, -1);
     g_strdown(text);
     gdict_app_do_lookup (text);
+    if (text)
+        g_free (text);
 }
 
 static void
@@ -228,19 +231,23 @@ lookup_defbox (void)
     gtk_signal_emit_by_name (GTK_OBJECT (defbox), "copy_clipboard");
     gtk_entry_set_text (GTK_ENTRY (word_entry), "");
     gtk_signal_emit_by_name (GTK_OBJECT (word_entry), "paste_clipboard");
-    text = gtk_entry_get_text (GTK_ENTRY (word_entry));
+    text = gtk_editable_get_chars (GTK_EDITABLE (word_entry), 0, -1);
     g_strdown (text);
     gnome_entry_append_history (GNOME_ENTRY (gnome_word_entry), 1, text);
     gdict_app_do_lookup (text);
+    if (text)
+        g_free (text);
 }
 
 static void
 lookup_any (void) 
 {
-    if (GTK_WIDGET_HAS_FOCUS (word_entry))
-	lookup_entry ();
-    else if (GTK_WIDGET_HAS_FOCUS (defbox))
+    /*if (GTK_WIDGET_HAS_FOCUS (word_entry))
+	lookup_entry ();*/
+    if (GTK_WIDGET_HAS_FOCUS (defbox))
 	lookup_defbox ();
+    else
+        lookup_entry ();
 }
 
 static void
@@ -285,14 +292,39 @@ lookup_cb (GtkWidget *menuitem, gpointer user_data)
     lookup_any();
 }
 
+static void
+lookup_button_cb (GtkButton *button, gpointer data)
+{
+    lookup_any();
+}
+
+static void
+lookup_button_drag_cb (GtkWidget *widget, GdkDragContext *context, gint x, gint y,
+		       GtkSelectionData *sd, guint info, guint t, gpointer data)
+{
+	gchar *text;
+	
+	text = gtk_selection_data_get_text (sd);
+	
+	if (text) {
+		g_strdown (text);
+		gtk_entry_set_text (GTK_ENTRY (word_entry), text);
+    		gnome_entry_append_history (GNOME_ENTRY (gnome_word_entry), 1, text);
+    		gdict_app_do_lookup (text);
+		g_free (text);
+	}
+}
+
 static void
 spell_cb (GtkWidget *menuitem, gpointer user_data) 
 {
     gchar *text;
 
-    text = gtk_entry_get_text (GTK_ENTRY (word_entry));
+    text = gtk_editable_get_chars (GTK_EDITABLE (word_entry), 0, -1);
     if (gdict_spell (text, FALSE) < 0)
 	gdict_not_online ();
+    if (text)
+        g_free (text);
 }
 
 #ifdef HAVE_GNOME_PRINT
@@ -347,14 +379,24 @@ paste_cb (GtkWidget *menuitem, gpointer user_data)
 static void
 clear_cb (GtkWidget *menuitem, gpointer user_data) 
 {
-    gtk_editable_select_region (GTK_EDITABLE (defbox), 0, 0);
+    gdict_defbox_clear (defbox);
 }
 
 static void
 select_all_cb (GtkMenuItem *menuitem, gpointer user_data) 
 {
-    gtk_editable_select_region (GTK_EDITABLE (defbox), 0, 
-                                gtk_text_get_length (GTK_TEXT (defbox)));
+    GtkTextBuffer* buffer = NULL;
+    GtkTextIter start_iter, end_iter;
+    
+    buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (defbox));
+    
+    gtk_text_buffer_get_bounds (buffer, &start_iter, &end_iter);
+    
+    gtk_text_buffer_place_cursor (buffer, &end_iter);
+    gtk_text_buffer_move_mark (buffer,
+                               gtk_text_buffer_get_mark (buffer, "selection_bound"),
+                               &start_iter);
+
 }
 
 static gchar *find_text;
@@ -362,13 +404,17 @@ static gchar *find_text;
 static void
 prompt_response_cb (GnomeAppBar *ab) 
 {
+    gchar *tmp;
+
+    tmp = gnome_appbar_get_response(ab);
     /* fprintf(stderr, "Response\n"); */
-    if (ab->prompt) {
+    if (tmp) {
         g_free(find_text);
         find_text = gnome_appbar_get_response(ab);
         if (find_text)
             gdict_defbox_find (defbox, find_text, TRUE);
         gnome_appbar_clear_prompt(GNOME_APPBAR(gdict_appbar));
+	g_free(tmp);
     }
 }
 
@@ -449,8 +495,8 @@ about_cb (GtkMenuItem *menuitem, gpointer user_data)
 }
 
 static GnomeUIInfo file_menu_uiinfo[] = {
-    GNOMEUIINFO_ITEM_STOCK (N_("Look up"),
-			    N_("Look up word in dictionary"),
+    GNOMEUIINFO_ITEM_STOCK (N_("Lookup"),
+			    N_("Lookup word in dictionary"),
 			    lookup_cb, GNOME_STOCK_MENU_SEARCH),
     GNOMEUIINFO_ITEM_STOCK (N_("Spell"), 
 			    N_("Check word spelling"), 
@@ -506,8 +552,8 @@ static GnomeUIInfo menubar_uiinfo[] = {
 };
 
 static GnomeUIInfo toolbar_uiinfo[] = {
-    GNOMEUIINFO_ITEM_STOCK (N_("Look up"), 
-			    N_("Look up word in dictionary"), 
+    GNOMEUIINFO_ITEM_STOCK (N_("Lookup"), 
+			    N_("Lookup word in dictionary"), 
 			    lookup_cb, GNOME_STOCK_PIXMAP_SEARCH),
     GNOMEUIINFO_ITEM_STOCK (N_("Spell"), 
 			    N_("Check word spelling"), 
@@ -529,22 +575,41 @@ static GnomeUIInfo toolbar_uiinfo[] = {
     GNOMEUIINFO_END
 };
 
-GtkWidget *gdict_app_create (void) {
+enum
+{
+  TARGET_STRING,
+  TARGET_TEXT,
+  TARGET_COMPOUND_TEXT,
+  TARGET_UTF8_STRING,
+  TARGET_TEXT_BUFFER_CONTENTS
+};
+
+GtkWidget *gdict_app_create (gboolean applet) {
     GtkWidget *dock;
-    GtkWidget *vbox, *hbox;
-    GtkAdjustment *vscrolladj;
+    GtkWidget *vbox;
+    GtkWidget *hbox;
+    GtkWidget *button;
+    GtkWidget *label;
+    GtkWidget *scrolled;
     GtkWidget *defbox_vscrollbar;
     GtkTooltips *tooltips;
+    static GtkTargetEntry drop_targets [] = {
+    	{ "UTF8_STRING", 0, 0 },
+  	{ "COMPOUND_TEXT", 0, 0 },
+  	{ "TEXT", 0, 0 },
+  	{ "text/plain", 0, 0 },
+  	{ "STRING",     0, 0 }
+    };
   
     tooltips = gtk_tooltips_new ();
   
     gdict_app = gnome_app_new ("GDict", "GDict");
-    gtk_widget_set_usize (gdict_app, 600, 330);
-  
+    gtk_window_set_default_size (GTK_WINDOW (gdict_app), 450, 330);
+    
     dock = GNOME_APP (gdict_app)->dock;
     gtk_widget_show (dock);
-
-    if (gdict_applet_toggle)  /* quick hack */
+    
+    if (applet)  /* quick hack */
         memcpy (&file_menu_uiinfo[EXIT_FILE_MENU_ITEM],
 		&applet_close_item, sizeof(GnomeUIInfo));
 
@@ -554,23 +619,46 @@ GtkWidget *gdict_app_create (void) {
     vbox = gtk_vbox_new (FALSE, 0);
     gtk_widget_show (vbox);
     gnome_app_set_contents (GNOME_APP (gdict_app), vbox);
+    
+    hbox = gtk_hbox_new (FALSE, GNOME_PAD_SMALL);
+    gtk_container_set_border_width (GTK_CONTAINER (hbox), GNOME_PAD_SMALL);
+    gtk_widget_show (hbox);
+    gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, FALSE, 0);
+    
+    label = gtk_label_new_with_mnemonic (_("_Lookup"));
+    gtk_widget_show (label);
+    gtk_misc_set_padding (GTK_MISC (label), 2, 1);
+    
+    button = gtk_button_new ();
+    gtk_container_add (GTK_CONTAINER (button), label);
+    gtk_widget_show (button);
+    gtk_box_pack_start (GTK_BOX (hbox), button, FALSE, FALSE, 0);
+    gtk_drag_dest_set (button, GTK_DEST_DEFAULT_ALL, drop_targets, 
+    		       G_N_ELEMENTS (drop_targets), GDK_ACTION_COPY | GDK_ACTION_MOVE);
+    g_signal_connect (G_OBJECT (button), "clicked",
+    		      G_CALLBACK (lookup_button_cb), defbox);
+    g_signal_connect (G_OBJECT (button), "drag_data_received",
+    		      G_CALLBACK (lookup_button_drag_cb), defbox);
 
     gnome_word_entry = gnome_entry_new ("Word entry");
     gtk_widget_show (gnome_word_entry);
-    gtk_box_pack_start (GTK_BOX (vbox), gnome_word_entry, FALSE, FALSE, 0);
+    /*gtk_box_pack_start (GTK_BOX (vbox), gnome_word_entry, FALSE, FALSE, 0);*/
+    gtk_box_pack_start (GTK_BOX (hbox), gnome_word_entry, TRUE, TRUE, 0);
     GTK_WIDGET_SET_FLAGS (gnome_word_entry, GTK_CAN_FOCUS);
     gtk_widget_grab_focus (gnome_word_entry);
     gnome_entry_set_max_saved (GNOME_ENTRY (gnome_word_entry), 50);
     word_entry = gnome_entry_gtk_entry(GNOME_ENTRY (gnome_word_entry));
-    hbox = gtk_hbox_new (FALSE, 0);
-    gtk_widget_show (hbox);
-    gtk_box_pack_start (GTK_BOX (vbox), hbox, TRUE, TRUE, 0);
-
-    vscrolladj = GTK_ADJUSTMENT(gtk_adjustment_new(0, 0, 0, 0, 0, 0));
-    defbox_vscrollbar = gtk_vscrollbar_new (vscrolladj);
-    gtk_widget_show (defbox_vscrollbar);
+        
+    scrolled = gtk_scrolled_window_new (NULL, NULL);
+    gtk_container_set_border_width (GTK_CONTAINER (scrolled), GNOME_PAD_SMALL);    
+    gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolled),
+                                        GTK_POLICY_AUTOMATIC,
+                                        GTK_POLICY_AUTOMATIC);
+    gtk_widget_show (scrolled);
   
     defbox = GDICT_DEFBOX (gdict_defbox_new ());
+    defbox_setup_tags (defbox);
+    gtk_text_view_set_left_margin (GTK_TEXT_VIEW (defbox), GNOME_PAD_SMALL);
     gtk_signal_connect (GTK_OBJECT (defbox), "word_lookup_start",
                         GTK_SIGNAL_FUNC (def_lookup_start_cb), defbox);
     gtk_signal_connect (GTK_OBJECT (defbox), "word_lookup_done",
@@ -582,18 +670,17 @@ GtkWidget *gdict_app_create (void) {
     gtk_signal_connect (GTK_OBJECT (defbox), "socket_error",
                         GTK_SIGNAL_FUNC (socket_error_cb), defbox);
   
-    gtk_text_set_adjustments (GTK_TEXT (defbox), NULL, vscrolladj);
     gtk_widget_show (GTK_WIDGET (defbox));
-    gtk_tooltips_set_tip (tooltips, GTK_WIDGET (defbox), 
-                          _("Word definition"), NULL);
-    gtk_box_pack_start (GTK_BOX (hbox), GTK_WIDGET (defbox), TRUE, TRUE, 0);
-    gtk_box_pack_start (GTK_BOX (hbox), defbox_vscrollbar, FALSE, TRUE, 0);
+    gtk_text_view_set_editable (GTK_TEXT_VIEW (defbox), FALSE);
+    gtk_container_add (GTK_CONTAINER (scrolled), GTK_WIDGET (defbox));
+
+    gtk_box_pack_start (GTK_BOX (vbox), scrolled, TRUE, TRUE, 0);
 
     gdict_appbar = gnome_appbar_new (TRUE, TRUE, GNOME_PREFERENCES_ALWAYS);
     gtk_widget_show (gdict_appbar);
     gnome_app_set_statusbar (GNOME_APP (gdict_app), gdict_appbar);
-    
-    if (!gdict_applet_toggle)
+   
+    if (!applet)
       gtk_signal_connect (GTK_OBJECT (gdict_app), "delete_event",
                           GTK_SIGNAL_FUNC (gtk_main_quit), NULL);
     else
