@@ -181,8 +181,25 @@ makecmd(char *start_dir)
 	}
 }
 
+static gboolean
+kill_after_nth_nl (GString *str, int n)
+{
+	int i;
+	int count = 0;
+	for (i = 0; str->str[i] != '\0'; i++) {
+		if (str->str[i] == '\n') {
+			count++;
+			if (count == n) {
+				g_string_truncate (str, i);
+				return TRUE;
+			}
+		}
+	}
+	return FALSE;
+}
+
 static void
-really_run_command(char *cmd, RunLevel *running)
+really_run_command(char *cmd, char sepchar, RunLevel *running)
 {
 	static gboolean lock = FALSE;
 	int idle;
@@ -193,6 +210,7 @@ really_run_command(char *cmd, RunLevel *running)
 	int i,n;
 	int pid;
 	GString *errors = NULL;
+	gboolean add_to_errors = TRUE;
 
 	if( ! lock) {
 		lock = TRUE;
@@ -244,7 +262,7 @@ really_run_command(char *cmd, RunLevel *running)
 	while (*running == RUNNING) {
 		n = read (fd[0], ret, PIPE_READ_BUFFER);
 		for (i = 0; i < n; i++) {
-			if(ret[i] == '\0') {
+			if(ret[i] == sepchar) {
 				outdlg_additem (string->str);
 				g_string_assign (string, "");
 			} else {
@@ -255,11 +273,15 @@ really_run_command(char *cmd, RunLevel *running)
 		n = read (fderr[0], ret, PIPE_READ_BUFFER-1);
 		if (n > 0) {
 			ret[n] = '\0';
-			if(!errors)
-				errors = g_string_new(ret);
-			else
-				errors = g_string_append(errors, ret);
-			fprintf(stderr, "%s", ret);
+			if (add_to_errors) {
+				if (errors == NULL)
+					errors = g_string_new (ret);
+				else
+					errors = g_string_append (errors, ret);
+				add_to_errors =
+					! kill_after_nth_nl (errors, 20);
+			}
+			fprintf (stderr, "%s", ret);
 		}
 		
 		if (waitpid (-1, NULL, WNOHANG) != 0)
@@ -277,7 +299,7 @@ really_run_command(char *cmd, RunLevel *running)
 	/* now we got it all ... so finish reading from the pipe */
 	while ((n = read (fd[0], ret, PIPE_READ_BUFFER)) > 0) {
 		for (i = 0; i < n; i++) {
-			if (ret[i] == '\0') {
+			if (ret[i] == sepchar) {
 				outdlg_additem (string->str);
 				g_string_assign (string, "");
 			} else {
@@ -287,11 +309,15 @@ really_run_command(char *cmd, RunLevel *running)
 	}
 	while((n = read(fderr[0], ret, PIPE_READ_BUFFER-1)) > 0) {
 		ret[n]='\0';
-		if(errors == NULL)
-			errors = g_string_new(ret);
-		else
-			errors = g_string_append(errors, ret);
-		fprintf(stderr, "%s", ret);
+		if (add_to_errors) {
+			if (errors == NULL)
+				errors = g_string_new (ret);
+			else
+				errors = g_string_append (errors, ret);
+			add_to_errors =
+				! kill_after_nth_nl (errors, 20);
+		}
+		fprintf (stderr, "%s", ret);
 	}
 
 	close(fd[0]);
@@ -304,10 +330,15 @@ really_run_command(char *cmd, RunLevel *running)
 	
 	/* if any errors occured */
 	if(errors) {
+		if ( ! add_to_errors) {
+			errors = g_string_append
+				(errors,
+				 _("\n... Too many errors to display ..."));
+		}
 		/* make an error message */
-		gnome_app_error(GNOME_APP(app), errors->str);
+		gnome_app_error (GNOME_APP (app), errors->str);
 		/* freeing allocated memory */
-		g_string_free(errors, TRUE);
+		g_string_free (errors, TRUE);
 	}
 
 	g_string_free (string, TRUE);
@@ -347,7 +378,7 @@ run_command(GtkWidget *w, gpointer data)
 	cmd = makecmd(start_dir);
 	g_free(start_dir);
 
-	really_run_command(cmd, &find_running);
+	really_run_command(cmd, '\0', &find_running);
 	g_free(cmd);
 
 	gtk_widget_set_sensitive(buttons[0], FALSE);
@@ -458,14 +489,14 @@ entry_changed(GtkWidget *w, gpointer data)
 	switch(templates[opt->templ].type) {
 	case FIND_OPTION_TEXT:
 	case FIND_OPTION_GREP:
-		opt->data.text = gtk_entry_get_text(GTK_ENTRY(w));
+		opt->data.text = (gchar *)gtk_entry_get_text(GTK_ENTRY(w));
 		break;
 	case FIND_OPTION_NUMBER:
 		sscanf(gtk_entry_get_text(GTK_ENTRY(w)),"%d",
 		       &opt->data.number);
 		break;
 	case FIND_OPTION_TIME:
-		opt->data.time = gtk_entry_get_text(GTK_ENTRY(w));
+		opt->data.time = (gchar *)gtk_entry_get_text(GTK_ENTRY(w));
 		break;
 	default:
 		g_warning(_("Entry changed called for a non entry option!"));
@@ -652,7 +683,7 @@ create_find_page(void)
 static void
 run_locate_command(GtkWidget *w, gpointer data)
 {
-	char *cmd;
+	gchar *cmd;
 	GtkWidget **buttons = data;
 
 	char *locate_string;
@@ -663,7 +694,7 @@ run_locate_command(GtkWidget *w, gpointer data)
 		return;
 	}
 
-	locate_string = gtk_entry_get_text(GTK_ENTRY(locate_entry));
+	locate_string = (gchar *)gtk_entry_get_text(GTK_ENTRY(locate_entry));
 	if(!locate_string || !*locate_string) {
 		gnome_app_error (GNOME_APP(app),
 				 _("Nothing to locate"));
@@ -678,13 +709,20 @@ run_locate_command(GtkWidget *w, gpointer data)
 
 	g_free(locate_string);
 
-	really_run_command(cmd, &locate_running);
+	really_run_command(cmd, '\n', &locate_running);
 	g_free(cmd);
 
 	gtk_widget_set_sensitive(buttons[0], FALSE);
 	gtk_widget_set_sensitive(buttons[1], TRUE);
 }
 
+static void
+locate_activate (GtkWidget *entry, gpointer data)
+{
+	GtkWidget **buttons = data;
+	run_locate_command (buttons[1], buttons);
+}
+
 static GtkWidget *
 create_locate_page(void)
 {
@@ -708,6 +746,9 @@ create_locate_page(void)
 
 	locate_entry = gtk_entry_new();
 	gtk_box_pack_start(GTK_BOX(hbox), locate_entry, TRUE, TRUE, 0);
+	gtk_signal_connect (GTK_OBJECT (locate_entry), "activate",
+			    GTK_SIGNAL_FUNC (locate_activate),
+			    buttons);
 
 	hbox = gtk_hbox_new(FALSE,GNOME_PAD_SMALL);
 	gtk_box_pack_end(GTK_BOX(vbox),hbox,FALSE,FALSE,0);
@@ -752,6 +793,11 @@ about_cb (GtkWidget *widget, gpointer data)
 		"George Lebl <jirka@5z.com>",
 		NULL
 	};
+	gchar *documenters[] = {
+		NULL
+	};
+	/* Translator credits */
+	gchar *translator_credits = _("");
 
 	if (about != NULL)
 	{
@@ -762,9 +808,11 @@ about_cb (GtkWidget *widget, gpointer data)
 
 	about = gnome_about_new(_("The Gnome Search Tool"), VERSION,
 				_("(C) 1998,2000 the Free Software Foundation"),
-				authors,
 				_("Frontend to the unix find/grep/locate "
 				  "commands"),
+				authors,
+				(const char **)documenters,
+				(const char *)translator_credits,
 				NULL);
 	gtk_signal_connect (GTK_OBJECT (about), "destroy",
 			    GTK_SIGNAL_FUNC (gtk_widget_destroyed), &about);
@@ -815,8 +863,9 @@ main(int argc, char *argv[])
 	GtkWidget *search;
 
 	/* Initialize the i18n stuff */
-	bindtextdomain (PACKAGE, GNOMELOCALEDIR);
-	textdomain (PACKAGE);
+	bindtextdomain(GETTEXT_PACKAGE, GNOMELOCALEDIR);
+	bind_textdomain_codeset(GETTEXT_PACKAGE, "UTF-8");
+	textdomain(GETTEXT_PACKAGE);
 
 	gnome_init ("gsearchtool", VERSION, argc, argv);
 	gnome_window_icon_set_default_from_file (GNOME_ICONDIR"/gnome-searchtool.png");
